"use strict";

/* eslint-disable no-console */

import { createHighlighterCore } from '@shikijs/core';
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript';
import React, { createContext, useCallback, useContext, useEffect, useMemo, useState } from 'react';
import { jsx as _jsx } from "react/jsx-runtime";
const ShikiContext = /*#__PURE__*/createContext(null);

// Default empty arrays as stable references
const DEFAULT_LANGUAGES = [];
const DEFAULT_THEMES = [];
const DEBUG = true;
function log(...args) {
  if (DEBUG) {
    console.log('[ShikiProvider]', ...args);
  }
}
export function ShikiProvider({
  children,
  initialLanguages = DEFAULT_LANGUAGES,
  initialThemes = DEFAULT_THEMES
}) {
  const [highlighter, setHighlighter] = useState(null);
  const [isReady, setIsReady] = useState(false);
  const [error, setError] = useState(null);
  useEffect(() => {
    const initializeHighlighter = async () => {
      try {
        log('Initializing highlighter...');
        const engine = createJavaScriptRegexEngine({
          forgiving: true,
          cache: new Map()
        });

        // Create the highlighter with explicit options
        const instance = await createHighlighterCore({
          engine,
          langs: initialLanguages,
          themes: initialThemes
        });

        // Test tokenization
        const {
          tokens: testTokens
        } = instance.codeToTokens('function test() { return true; }', {
          lang: 'typescript',
          theme: 'github-dark',
          includeExplanation: true
        });
        log('Test tokenization:', {
          tokens: testTokens[0]?.map(t => ({
            content: t.content,
            color: t.color,
            scopes: t.explanation?.[0]?.scopes
          }))
        });
        setHighlighter(instance);
        setIsReady(true);
      } catch (err) {
        console.error('Full error:', err);
        setError(err instanceof Error ? err : new Error('Failed to initialize highlighter'));
      }
    };
    initializeHighlighter();
  }, [initialLanguages, initialThemes]);
  const registerLanguage = useCallback(async lang => {
    if (!highlighter) {
      log('Cannot register language, highlighter not ready:', lang.name);
      return;
    }
    try {
      log('Registering language:', lang.name);
      await highlighter.loadLanguage(lang);
      log('Language registered:', lang.name);
    } catch (err) {
      log('Error registering language:', lang.name, err);
    }
  }, [highlighter]);
  const registerTheme = useCallback(async theme => {
    if (!highlighter) {
      log('Cannot register theme, highlighter not ready:', typeof theme === 'string' ? theme : theme.name);
      return;
    }
    try {
      log('Registering theme:', typeof theme === 'string' ? theme : theme.name);
      await highlighter.loadTheme(theme);
      log('Theme registered:', typeof theme === 'string' ? theme : theme.name);
    } catch (err) {
      log('Error registering theme:', typeof theme === 'string' ? theme : theme.name, err);
    }
  }, [highlighter]);
  const value = useMemo(() => ({
    highlighter,
    isReady,
    error,
    registerLanguage,
    registerTheme,
    getLoadedLanguages: () => {
      const langs = highlighter?.getLoadedLanguages() || [];
      log('Getting loaded languages:', langs);
      return langs;
    },
    getLoadedThemes: () => {
      const themes = highlighter?.getLoadedThemes() || [];
      log('Getting loaded themes:', themes);
      return themes;
    }
  }), [highlighter, isReady, error, registerLanguage, registerTheme]);
  useEffect(() => {
    log('ShikiProvider mounted');
    return () => log('ShikiProvider unmounted');
  }, []);
  return /*#__PURE__*/_jsx(ShikiContext.Provider, {
    value: value,
    children: children
  });
}

// Add logging to hook usage
export function useShiki() {
  const context = useContext(ShikiContext);
  if (!context) {
    log('useShiki called outside of provider!');
    throw new Error('useShiki must be used within a ShikiProvider');
  }
  return context;
}
//# sourceMappingURL=SyntaxContext.js.map