"use strict";

/* eslint-disable no-console */

import React, { memo, useCallback, useEffect, useState } from 'react';
import { ActivityIndicator, ScrollView, StyleSheet, Text, View } from 'react-native';
import { getStylesFromShikiTheme } from '../utils/style';
import { useShiki } from './SyntaxContext';
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const DEBUG = true;
function log(...args) {
  if (DEBUG) {
    console.log('[SyntaxHighlighter]', ...args);
  }
}
const styles = StyleSheet.create({
  container: {
    flex: 1
  },
  content: {
    padding: 10
  },
  line: {
    flexDirection: 'row'
  },
  lineContent: {
    flexDirection: 'row',
    flexWrap: 'wrap'
  },
  lineNumber: {
    width: 40,
    paddingRight: 8,
    opacity: 0.5
  },
  lineNumberText: {
    textAlign: 'right'
  },
  error: {
    color: '#ff6b6b'
  },
  bold: {
    fontWeight: 'bold'
  },
  italic: {
    fontStyle: 'italic'
  },
  underline: {
    textDecorationLine: 'underline'
  }
});
export const SyntaxHighlighter = /*#__PURE__*/memo(({
  code,
  language,
  theme,
  fontSize = 14,
  textStyle,
  scrollViewProps,
  containerStyle,
  showLineNumbers = false,
  renderLineNumber,
  loadingComponent,
  errorComponent
}) => {
  const {
    highlighter,
    isReady
  } = useShiki();
  const [tokens, setTokens] = useState([]);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState(null);

  // Only get root styles from theme
  const stylesheet = React.useMemo(() => {
    const styles = getStylesFromShikiTheme(theme);
    log('Generated stylesheet from theme:', {
      themeName: typeof theme === 'string' ? theme : theme.name,
      rootStyle: styles.root
    });
    return styles;
  }, [theme]);
  const highlight = useCallback(async () => {
    if (!highlighter || !isReady) return;
    try {
      setLoading(true);
      setError(null);

      // Get theme name safely
      const themeName = typeof theme === 'string' ? theme : theme.name || 'default';

      // Convert language to string if it's a LanguageRegistration
      const langName = typeof language === 'string' ? language : language.name;

      // Ensure theme is loaded
      if (!highlighter.getLoadedThemes().includes(themeName)) {
        // Cast theme to ThemeRegistrationAny when loading
        await highlighter.loadTheme(typeof theme === 'string' ? {
          name: theme
        } : theme);
      }

      // Use codeToTokens with proper typing
      const result = highlighter.codeToTokens(code, {
        lang: langName,
        theme: themeName,
        includeExplanation: true
      });
      log('Tokenization result:', {
        lines: result.tokens.length,
        firstLineTokens: result.tokens[0]?.length,
        firstLineDetails: result.tokens[0]?.map(t => ({
          content: t.content,
          color: t.color,
          fontStyle: t.fontStyle,
          scopes: t.explanation?.[0]?.scopes
        }))
      });
      setTokens(result.tokens);
    } catch (err) {
      console.error('Full highlighting error:', err);
      setError(err instanceof Error ? err.message : 'Failed to highlight code');
    } finally {
      setLoading(false);
    }
  }, [code, language, theme, highlighter, isReady]);
  useEffect(() => {
    highlight();
  }, [highlight]);
  const renderToken = useCallback((token, lineIndex, tokenIndex) => {
    // Get the token's color and font style from Shiki
    const tokenStyle = {
      color: token.color,
      // Use the color directly from Shiki
      fontSize,
      ...(token.fontStyle && token.fontStyle & 1 && {
        fontStyle: 'italic'
      }),
      ...(token.fontStyle && token.fontStyle & 2 && {
        fontWeight: 'bold'
      }),
      ...(token.fontStyle && token.fontStyle & 4 && {
        textDecorationLine: 'underline'
      })
    };
    if (DEBUG && lineIndex === 0 && tokenIndex === 0) {
      log('Rendering token:', {
        content: token.content,
        color: token.color,
        fontStyle: token.fontStyle,
        computedStyle: tokenStyle
      });
    }
    return /*#__PURE__*/_jsx(Text, {
      style: [tokenStyle, textStyle],
      selectable: false,
      children: token.content
    }, `${lineIndex}-${tokenIndex}`);
  }, [fontSize, textStyle]);
  const renderLine = useCallback((lineTokens, lineIndex) => {
    if (DEBUG && lineIndex === 0) {
      log('Rendering first line:', {
        tokenCount: lineTokens.length,
        tokens: lineTokens.map(t => ({
          content: t.content,
          color: t.color
        }))
      });
    }
    return /*#__PURE__*/_jsxs(View, {
      style: styles.line,
      children: [showLineNumbers && /*#__PURE__*/_jsx(View, {
        style: styles.lineNumber,
        children: renderLineNumber ? renderLineNumber(lineIndex + 1) : /*#__PURE__*/_jsx(Text, {
          style: styles.lineNumberText,
          children: (lineIndex + 1).toString().padStart(3, ' ')
        })
      }), /*#__PURE__*/_jsx(View, {
        style: styles.lineContent,
        children: lineTokens.map((token, tokenIndex) => renderToken(token, lineIndex, tokenIndex))
      })]
    }, lineIndex);
  }, [renderToken, showLineNumbers, renderLineNumber]);
  useEffect(() => {
    log('Component mounted with props:', {
      language,
      theme: typeof theme === 'string' ? theme : theme.name,
      isReady,
      hasHighlighter: !!highlighter
    });
    return () => log('Component unmounted');
  }, [language, theme, isReady, highlighter]);
  if (loading) {
    return loadingComponent || /*#__PURE__*/_jsx(View, {
      style: [styles.container, {
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/_jsx(ActivityIndicator, {})
    });
  }
  if (error) {
    return errorComponent?.(error) || /*#__PURE__*/_jsx(View, {
      style: [styles.container, {
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/_jsxs(Text, {
        style: styles.error,
        children: ["Error:", error]
      })
    });
  }
  const combinedStyle = [styles.container, stylesheet.root, scrollViewProps?.style];
  log('Rendering with styles:', {
    containerStyle: combinedStyle,
    tokenCount: tokens.reduce((acc, line) => acc + line.length, 0)
  });
  return /*#__PURE__*/_jsx(ScrollView, {
    ...scrollViewProps,
    style: combinedStyle,
    contentContainerStyle: [styles.content, containerStyle],
    horizontal: true,
    children: /*#__PURE__*/_jsx(View, {
      children: tokens.map((lineTokens, index) => renderLine(lineTokens, index))
    })
  });
});
//# sourceMappingURL=SyntaxHighlighter.js.map