/* eslint-disable no-console */
import type { LanguageInput, LanguageRegistration, SpecialLanguage, StringLiteralUnion, ThemedToken, ThemeRegistrationAny } from '@shikijs/types'
import React, { memo, useCallback, useEffect, useState } from 'react'
import { ActivityIndicator, ScrollView, type ScrollViewProps, type StyleProp, StyleSheet, Text, type TextStyle, View, type ViewStyle } from 'react-native'
import { getStylesFromShikiTheme } from '../utils/style'
import { useShiki } from './SyntaxContext'

const DEBUG = true
function log(...args: any[]) {
  if (DEBUG) {
    console.log('[SyntaxHighlighter]', ...args)
  }
}

interface SyntaxHighlighterProps {
  code: string
  language: LanguageInput | SpecialLanguage
  theme: ThemeRegistrationAny | StringLiteralUnion<string>
  fontSize?: number
  textStyle?: StyleProp<TextStyle>
  scrollViewProps?: ScrollViewProps
  containerStyle?: StyleProp<ViewStyle>
  showLineNumbers?: boolean
  renderLineNumber?: (lineNumber: number) => React.ReactNode
  loadingComponent?: React.ReactNode
  errorComponent?: (error: string) => React.ReactNode
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
  },
  content: {
    padding: 10,
  },
  line: {
    flexDirection: 'row',
  },
  lineContent: {
    flexDirection: 'row',
    flexWrap: 'wrap',
  },
  lineNumber: {
    width: 40,
    paddingRight: 8,
    opacity: 0.5,
  },
  lineNumberText: {
    textAlign: 'right',
  },
  error: {
    color: '#ff6b6b',
  },
  bold: {
    fontWeight: 'bold',
  },
  italic: {
    fontStyle: 'italic',
  },
  underline: {
    textDecorationLine: 'underline',
  },
})

export const SyntaxHighlighter = memo(
  ({ code, language, theme, fontSize = 14, textStyle, scrollViewProps, containerStyle, showLineNumbers = false, renderLineNumber, loadingComponent, errorComponent }: SyntaxHighlighterProps) => {
    const { highlighter, isReady } = useShiki()
    const [tokens, setTokens] = useState<ThemedToken[][]>([])
    const [loading, setLoading] = useState(true)
    const [error, setError] = useState<string | null>(null)

    // Only get root styles from theme
    const stylesheet = React.useMemo(() => {
      const styles = getStylesFromShikiTheme(theme)
      log('Generated stylesheet from theme:', {
        themeName: typeof theme === 'string' ? theme : theme.name,
        rootStyle: styles.root,
      })
      return styles
    }, [theme])

    const highlight = useCallback(async () => {
      if (!highlighter || !isReady)
        return

      try {
        setLoading(true)
        setError(null)

        // Get theme name safely
        const themeName = typeof theme === 'string'
          ? theme as string
          : (theme as ThemeRegistrationAny).name || 'default'

        // Convert language to string if it's a LanguageRegistration
        const langName = typeof language === 'string'
          ? language
          : (language as LanguageRegistration).name

        // Ensure theme is loaded
        if (!highlighter.getLoadedThemes().includes(themeName)) {
          // Cast theme to ThemeRegistrationAny when loading
          await highlighter.loadTheme(
            typeof theme === 'string'
              ? { name: theme } as ThemeRegistrationAny
              : theme as ThemeRegistrationAny,
          )
        }

        // Use codeToTokens with proper typing
        const result = highlighter.codeToTokens(code, {
          lang: langName,
          theme: themeName,
          includeExplanation: true,
        })

        log('Tokenization result:', {
          lines: result.tokens.length,
          firstLineTokens: result.tokens[0]?.length,
          firstLineDetails: result.tokens[0]?.map(t => ({
            content: t.content,
            color: t.color,
            fontStyle: t.fontStyle,
            scopes: t.explanation?.[0]?.scopes,
          })),
        })

        setTokens(result.tokens)
      }
      catch (err) {
        console.error('Full highlighting error:', err)
        setError(err instanceof Error ? err.message : 'Failed to highlight code')
      }
      finally {
        setLoading(false)
      }
    }, [code, language, theme, highlighter, isReady])

    useEffect(() => {
      highlight()
    }, [highlight])

    const renderToken = useCallback(
      (token: ThemedToken, lineIndex: number, tokenIndex: number) => {
        // Get the token's color and font style from Shiki
        const tokenStyle: TextStyle = {
          color: token.color, // Use the color directly from Shiki
          fontSize,
          ...(token.fontStyle && token.fontStyle & 1 && { fontStyle: 'italic' }),
          ...(token.fontStyle && token.fontStyle & 2 && { fontWeight: 'bold' }),
          ...(token.fontStyle && token.fontStyle & 4 && { textDecorationLine: 'underline' }),
        }

        if (DEBUG && lineIndex === 0 && tokenIndex === 0) {
          log('Rendering token:', {
            content: token.content,
            color: token.color,
            fontStyle: token.fontStyle,
            computedStyle: tokenStyle,
          })
        }

        return (
          <Text key={`${lineIndex}-${tokenIndex}`} style={[tokenStyle, textStyle]} selectable={false}>
            {token.content}
          </Text>
        )
      },
      [fontSize, textStyle],
    )

    const renderLine = useCallback(
      (lineTokens: ThemedToken[], lineIndex: number) => {
        if (DEBUG && lineIndex === 0) {
          log('Rendering first line:', {
            tokenCount: lineTokens.length,
            tokens: lineTokens.map(t => ({ content: t.content, color: t.color })),
          })
        }

        return (
          <View key={lineIndex} style={styles.line}>
            {showLineNumbers && (
              <View style={styles.lineNumber}>{renderLineNumber ? renderLineNumber(lineIndex + 1) : <Text style={styles.lineNumberText}>{(lineIndex + 1).toString().padStart(3, ' ')}</Text>}</View>
            )}
            <View style={styles.lineContent}>{lineTokens.map((token, tokenIndex) => renderToken(token, lineIndex, tokenIndex))}</View>
          </View>
        )
      },
      [renderToken, showLineNumbers, renderLineNumber],
    )

    useEffect(() => {
      log('Component mounted with props:', {
        language,
        theme: typeof theme === 'string' ? theme : theme.name,
        isReady,
        hasHighlighter: !!highlighter,
      })
      return () => log('Component unmounted')
    }, [language, theme, isReady, highlighter])

    if (loading) {
      return (
        loadingComponent || (
          <View style={[styles.container, { padding: 10 }, containerStyle]}>
            <ActivityIndicator />
          </View>
        )
      )
    }

    if (error) {
      return (
        errorComponent?.(error) || (
          <View style={[styles.container, { padding: 10 }, containerStyle]}>
            <Text style={styles.error}>
              Error:
              {error}
            </Text>
          </View>
        )
      )
    }

    const combinedStyle = [styles.container, stylesheet.root, scrollViewProps?.style]
    log('Rendering with styles:', {
      containerStyle: combinedStyle,
      tokenCount: tokens.reduce((acc, line) => acc + line.length, 0),
    })

    return (
      <ScrollView {...scrollViewProps} style={combinedStyle} contentContainerStyle={[styles.content, containerStyle]} horizontal>
        <View>{tokens.map((lineTokens, index) => renderLine(lineTokens, index))}</View>
      </ScrollView>
    )
  },
)

export type { SyntaxHighlighterProps }
